@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2006> Philippe Khalaf <burger@speedy.org> 
+ * Copyright (C) <2006> Philippe Khalaf <philippe.kalaf@collabora.co.uk>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -34,19 +34,19 @@
  * payloading is done based on the maximum MTU (mtu) and the maximum time per
  * packet (max-ptime). The general idea is to divide large data buffers into
  * smaller RTP packets. The RTP packet size is the minimum of either the MTU,
- * max-ptime (if set) or available data. Any residual data is always sent in a
- * last RTP packet (no minimum RTP packet size). A minimum packet size might be
- * added in future versions if the need arises. In the case of frame
- * based codecs, the resulting RTP packets always contain full frames.
+ * max-ptime (if set) or available data. The RTP packet size is always larger or
+ * equal to min-ptime (if set). If min-ptime is not set, any residual data is
+ * sent in a last RTP packet. In the case of frame based codecs, the resulting
+ * RTP packets always contain full frames.
  * </para>
  * <title>Usage</title>
  * <para>
  * To use this base class, your child element needs to call either
- * gst_basertpaudiopayload_set_frame_based() or
- * gst_basertpaudiopayload_set_sample_based(). This is usually done in the
+ * gst_base_rtp_audio_payload_set_frame_based() or
+ * gst_base_rtp_audio_payload_set_sample_based(). This is usually done in the
  * element's _init() function. Then, the child element must call either
- * gst_basertpaudiopayload_set_frame_options() or
- * gst_basertpaudiopayload_set_sample_options(). Since GstBaseRTPAudioPayload
+ * gst_base_rtp_audio_payload_set_frame_options() or
+ * gst_base_rtp_audio_payload_set_sample_options(). Since GstBaseRTPAudioPayload
  * derives from GstBaseRTPPayload, the child element must set any variables or
  * call/override any functions required by that base class. The child element
  * does not need to override any other functions specific to
@@ -62,6 +62,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <gst/rtp/gstrtpbuffer.h>
+#include <gst/base/gstadapter.h>
 
 #include "gstbasertpaudiopayload.h"
 
@@ -78,18 +79,17 @@ typedef enum
 struct _GstBaseRTPAudioPayloadPrivate
 {
   AudioCodecType type;
+  GstAdapter *adapter;
+  guint64 min_ptime;
 };
 
+
 #define GST_BASE_RTP_AUDIO_PAYLOAD_GET_PRIVATE(o) \
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), GST_TYPE_BASE_RTP_AUDIO_PAYLOAD, \
                                 GstBaseRTPAudioPayloadPrivate))
 
 static void gst_base_rtp_audio_payload_finalize (GObject * object);
 
-static GstFlowReturn
-gst_base_rtp_audio_payload_push (GstBaseRTPPayload * basepayload, guint8 * data,
-    guint payload_len, GstClockTime timestamp);
-
 static GstFlowReturn gst_base_rtp_audio_payload_handle_buffer (GstBaseRTPPayload
     * payload, GstBuffer * buffer);
 
@@ -101,6 +101,12 @@ static GstFlowReturn
 gst_base_rtp_audio_payload_handle_sample_based_buffer (GstBaseRTPPayload *
     basepayload, GstBuffer * buffer);
 
+static GstStateChangeReturn
+gst_base_rtp_payload_audio_change_state (GstElement * element,
+    GstStateChange transition);
+static gboolean
+gst_base_rtp_payload_audio_handle_event (GstPad * pad, GstEvent * event);
+
 GST_BOILERPLATE (GstBaseRTPAudioPayload, gst_base_rtp_audio_payload,
     GstBaseRTPPayload, GST_TYPE_BASE_RTP_PAYLOAD);
 
@@ -121,13 +127,17 @@ gst_base_rtp_audio_payload_class_init (GstBaseRTPAudioPayloadClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
   gstbasertppayload_class = (GstBaseRTPPayloadClass *) klass;
+
   gobject_class->finalize =
       GST_DEBUG_FUNCPTR (gst_base_rtp_audio_payload_finalize);
 
-  parent_class = g_type_class_ref (GST_TYPE_BASE_RTP_PAYLOAD);
+  gstelement_class->change_state =
+      GST_DEBUG_FUNCPTR (gst_base_rtp_payload_audio_change_state);
 
   gstbasertppayload_class->handle_buffer =
       GST_DEBUG_FUNCPTR (gst_base_rtp_audio_payload_handle_buffer);
+  gstbasertppayload_class->handle_event =
+      GST_DEBUG_FUNCPTR (gst_base_rtp_payload_audio_handle_event);
 
   GST_DEBUG_CATEGORY_INIT (basertpaudiopayload_debug, "basertpaudiopayload", 0,
       "base audio RTP payloader");
@@ -150,6 +160,8 @@ gst_base_rtp_audio_payload_init (GstBaseRTPAudioPayload * basertpaudiopayload,
 
   /* these need to be set by child object if sample based */
   basertpaudiopayload->sample_size = 0;
+
+  basertpaudiopayload->priv->adapter = gst_adapter_new ();
 }
 
 static void
@@ -159,6 +171,8 @@ gst_base_rtp_audio_payload_finalize (GObject * object)
 
   basertpaudiopayload = GST_BASE_RTP_AUDIO_PAYLOAD (object);
 
+  g_object_unref (basertpaudiopayload->priv->adapter);
+
   GST_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
@@ -203,7 +217,7 @@ gst_base_rtp_audio_payload_set_sample_based (GstBaseRTPAudioPayload *
 /**
  * gst_base_rtp_audio_payload_set_frame_options:
  * @basertpaudiopayload: a pointer to the element.
- * @frame_duration: The duraction of an audio frame in milliseconds. 
+ * @frame_duration: The duraction of an audio frame in milliseconds.
  * @frame_size: The size of an audio frame in bytes.
  *
  * Sets the options for frame based audio codecs.
@@ -217,6 +231,10 @@ gst_base_rtp_audio_payload_set_frame_options (GstBaseRTPAudioPayload
 
   basertpaudiopayload->frame_size = frame_size;
   basertpaudiopayload->frame_duration = frame_duration;
+
+  if (basertpaudiopayload->priv->adapter) {
+    gst_adapter_clear (basertpaudiopayload->priv->adapter);
+  }
 }
 
 /**
@@ -234,6 +252,10 @@ gst_base_rtp_audio_payload_set_sample_options (GstBaseRTPAudioPayload
   g_return_if_fail (basertpaudiopayload != NULL);
 
   basertpaudiopayload->sample_size = sample_size;
+
+  if (basertpaudiopayload->priv->adapter) {
+    gst_adapter_clear (basertpaudiopayload->priv->adapter);
+  }
 }
 
 static GstFlowReturn
@@ -267,14 +289,19 @@ gst_base_rtp_audio_payload_handle_frame_based_buffer (GstBaseRTPPayload *
 {
   GstBaseRTPAudioPayload *basertpaudiopayload;
   guint payload_len;
-  guint8 *data;
+  const guint8 *data = NULL;
   GstFlowReturn ret;
   guint available;
   gint frame_size, frame_duration;
 
   guint maxptime_octets = G_MAXUINT;
+  guint minptime_octets = 0;
+  guint min_payload_len;
+  guint max_payload_len;
+  gboolean use_adapter = FALSE;
+  guint minptime_ms;
 
-  ret = GST_FLOW_ERROR;
+  ret = GST_FLOW_OK;
 
   basertpaudiopayload = GST_BASE_RTP_AUDIO_PAYLOAD (basepayload);
 
@@ -287,19 +314,6 @@ gst_base_rtp_audio_payload_handle_frame_based_buffer (GstBaseRTPPayload *
   frame_size = basertpaudiopayload->frame_size;
   frame_duration = basertpaudiopayload->frame_duration;
 
-  /* If buffer fits on an RTP packet, let's just push it through */
-  /* this will check against max_ptime and max_mtu */
-  if (!gst_basertppayload_is_filled (basepayload,
-          gst_rtp_buffer_calc_packet_len (GST_BUFFER_SIZE (buffer), 0, 0),
-          GST_BUFFER_DURATION (buffer))) {
-    ret = gst_base_rtp_audio_payload_push (basepayload,
-        GST_BUFFER_DATA (buffer), GST_BUFFER_SIZE (buffer),
-        GST_BUFFER_TIMESTAMP (buffer));
-    gst_buffer_unref (buffer);
-
-    return ret;
-  }
-
   /* max number of bytes based on given ptime, has to be multiple of
    * frame_duration */
   if (basepayload->max_ptime != -1) {
@@ -313,45 +327,94 @@ gst_base_rtp_audio_payload_handle_frame_based_buffer (GstBaseRTPPayload *
     }
   }
 
-  /* let's set the base timestamp */
-  basertpaudiopayload->base_ts = GST_BUFFER_TIMESTAMP (buffer);
+  max_payload_len = MIN (
+      /* MTU max */
+      (int) (gst_rtp_buffer_calc_payload_len (GST_BASE_RTP_PAYLOAD_MTU
+              (basertpaudiopayload), 0, 0) / frame_size) * frame_size,
+      /* ptime max */
+      maxptime_octets);
 
-  available = GST_BUFFER_SIZE (buffer);
-  data = (guint8 *) GST_BUFFER_DATA (buffer);
+  /* min number of bytes based on a given ptime, has to be a multiple
+     of frame duration */
+  minptime_ms = basepayload->min_ptime / 1000000;
 
-  /* as long as we have full frames */
-  /* this loop will push all available buffers till the last frame */
-  while (available >= frame_size) {
-    /* we need to see how many frames we can get based on maximum MTU, maximum
-     * ptime and the number of bytes available */
-    payload_len = MIN (MIN (
-            /* MTU max */
-            (int) (gst_rtp_buffer_calc_payload_len (GST_BASE_RTP_PAYLOAD_MTU
-                    (basertpaudiopayload), 0, 0) / frame_size) * frame_size,
-            /* ptime max */
-            maxptime_octets),
-        /* currently available */
-        (available / frame_size) * frame_size);
+  minptime_octets = frame_size * (int) (minptime_ms / frame_duration);
 
-    ret = gst_base_rtp_audio_payload_push (basepayload, data, payload_len,
-        basertpaudiopayload->base_ts);
+  min_payload_len = MAX (minptime_octets, frame_size);
 
-    gfloat ts_inc = (payload_len * frame_duration) / frame_size;
-
-    ts_inc = ts_inc * GST_MSECOND;
-    basertpaudiopayload->base_ts += ts_inc;
-
-    available -= payload_len;
-    data += payload_len;
+  if (min_payload_len > max_payload_len) {
+    min_payload_len = max_payload_len;
   }
 
-  gst_buffer_unref (buffer);
+  GST_DEBUG_OBJECT (basertpaudiopayload,
+      "Calculated min_payload_len %u and max_payload_len %u",
+      min_payload_len, max_payload_len);
 
-  /* none should be available by now */
-  if (available != 0) {
-    GST_ERROR_OBJECT (basertpaudiopayload, "The buffer size is not a multiple"
-        " of the frame_size");
-    return GST_FLOW_ERROR;
+  if (basertpaudiopayload->priv->adapter &&
+      gst_adapter_available (basertpaudiopayload->priv->adapter)) {
+    /* If there is always data in the adapter, we have to use it */
+    gst_adapter_push (basertpaudiopayload->priv->adapter, buffer);
+    available = gst_adapter_available (basertpaudiopayload->priv->adapter);
+    use_adapter = TRUE;
+  } else {
+    /* let's set the base timestamp */
+    basertpaudiopayload->base_ts = GST_BUFFER_TIMESTAMP (buffer);
+
+    /* If buffer fits on an RTP packet, let's just push it through */
+    /* this will check against max_ptime and max_mtu */
+    if (GST_BUFFER_SIZE (buffer) >= min_payload_len &&
+        GST_BUFFER_SIZE (buffer) <= max_payload_len) {
+      ret = gst_base_rtp_audio_payload_push (basertpaudiopayload,
+          GST_BUFFER_DATA (buffer), GST_BUFFER_SIZE (buffer),
+          GST_BUFFER_TIMESTAMP (buffer));
+      gst_buffer_unref (buffer);
+
+      return ret;
+    }
+
+    available = GST_BUFFER_SIZE (buffer);
+    data = (guint8 *) GST_BUFFER_DATA (buffer);
+  }
+
+  /* as long as we have full frames */
+  while (available >= min_payload_len) {
+    gfloat ts_inc;
+
+    /* We send as much as we can */
+    payload_len = MIN (max_payload_len, (available / frame_size) * frame_size);
+
+    if (use_adapter) {
+      data = gst_adapter_peek (basertpaudiopayload->priv->adapter, payload_len);
+    }
+
+    ret =
+        gst_base_rtp_audio_payload_push (basertpaudiopayload, data, payload_len,
+        basertpaudiopayload->base_ts);
+
+    ts_inc = (payload_len * frame_duration) / frame_size;
+
+    ts_inc = ts_inc * GST_MSECOND;
+    basertpaudiopayload->base_ts += gst_gdouble_to_guint64 (ts_inc);
+
+    if (use_adapter) {
+      gst_adapter_flush (basertpaudiopayload->priv->adapter, payload_len);
+      available = gst_adapter_available (basertpaudiopayload->priv->adapter);
+    } else {
+      available -= payload_len;
+      data += payload_len;
+    }
+  }
+
+  if (!use_adapter) {
+    if (available != 0 && basertpaudiopayload->priv->adapter) {
+      GstBuffer *buf;
+
+      buf = gst_buffer_create_sub (buffer,
+          GST_BUFFER_SIZE (buffer) - available, available);
+      gst_adapter_push (basertpaudiopayload->priv->adapter, buf);
+    } else {
+      gst_buffer_unref (buffer);
+    }
   }
 
   return ret;
@@ -363,15 +426,19 @@ gst_base_rtp_audio_payload_handle_sample_based_buffer (GstBaseRTPPayload *
 {
   GstBaseRTPAudioPayload *basertpaudiopayload;
   guint payload_len;
-  guint8 *data;
+  const guint8 *data = NULL;
   GstFlowReturn ret;
   guint available;
 
   guint maxptime_octets = G_MAXUINT;
+  guint minptime_octets = 0;
+  guint min_payload_len;
+  guint max_payload_len;
+  gboolean use_adapter = FALSE;
 
   guint sample_size;
 
-  ret = GST_FLOW_ERROR;
+  ret = GST_FLOW_OK;
 
   basertpaudiopayload = GST_BASE_RTP_AUDIO_PAYLOAD (basepayload);
 
@@ -382,79 +449,134 @@ gst_base_rtp_audio_payload_handle_sample_based_buffer (GstBaseRTPPayload *
   }
   sample_size = basertpaudiopayload->sample_size;
 
-  /* If buffer fits on an RTP packet, let's just push it through */
-  /* this will check against max_ptime and max_mtu */
-  if (!gst_basertppayload_is_filled (basepayload,
-          gst_rtp_buffer_calc_packet_len (GST_BUFFER_SIZE (buffer), 0, 0),
-          GST_BUFFER_DURATION (buffer))) {
-    ret = gst_base_rtp_audio_payload_push (basepayload,
-        GST_BUFFER_DATA (buffer), GST_BUFFER_SIZE (buffer),
-        GST_BUFFER_TIMESTAMP (buffer));
-    gst_buffer_unref (buffer);
-
-    return ret;
-  }
-
   /* max number of bytes based on given ptime */
   if (basepayload->max_ptime != -1) {
     maxptime_octets = basepayload->max_ptime * basepayload->clock_rate /
         (sample_size * GST_SECOND);
-    GST_DEBUG_OBJECT (basertpaudiopayload, "Calculated max_octects %u",
-        maxptime_octets);
   }
 
-  /* let's set the base timestamp */
-  basertpaudiopayload->base_ts = GST_BUFFER_TIMESTAMP (buffer);
-  GST_DEBUG_OBJECT (basertpaudiopayload, "Setting to %" GST_TIME_FORMAT,
-      GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)));
+  max_payload_len = MIN (
+      /* MTU max */
+      gst_rtp_buffer_calc_payload_len (GST_BASE_RTP_PAYLOAD_MTU
+          (basertpaudiopayload), 0, 0),
+      /* ptime max */
+      maxptime_octets);
 
-  available = GST_BUFFER_SIZE (buffer);
-  data = (guint8 *) GST_BUFFER_DATA (buffer);
+  /* min number of bytes based on a given ptime, has to be a multiple
+     of sample rate */
+  minptime_octets = basepayload->min_ptime * basepayload->clock_rate /
+      (sample_size * GST_SECOND);
 
-  /* as long as we have full frames */
-  /* this loop will use all available data until the last byte */
-  while (available) {
-    /* we need to see how many frames we can get based on maximum MTU, maximum
-     * ptime and the number of bytes available */
-    payload_len = MIN (MIN (
-            /* MTU max */
-            gst_rtp_buffer_calc_payload_len (GST_BASE_RTP_PAYLOAD_MTU
-                (basertpaudiopayload), 0, 0),
-            /* ptime max */
-            maxptime_octets),
-        /* currently available */
-        available);
+  min_payload_len = MAX (minptime_octets, sample_size);
 
-    ret = gst_base_rtp_audio_payload_push (basepayload, data, payload_len,
+  if (min_payload_len > max_payload_len) {
+    min_payload_len = max_payload_len;
+  }
+
+  GST_DEBUG_OBJECT (basertpaudiopayload,
+      "Calculated min_payload_len %u and max_payload_len %u",
+      min_payload_len, max_payload_len);
+
+  if (basertpaudiopayload->priv->adapter &&
+      gst_adapter_available (basertpaudiopayload->priv->adapter)) {
+    /* If there is always data in the adapter, we have to use it */
+    gst_adapter_push (basertpaudiopayload->priv->adapter, buffer);
+    available = gst_adapter_available (basertpaudiopayload->priv->adapter);
+    use_adapter = TRUE;
+  } else {
+    /* let's set the base timestamp */
+    basertpaudiopayload->base_ts = GST_BUFFER_TIMESTAMP (buffer);
+
+    /* If buffer fits on an RTP packet, let's just push it through */
+    /* this will check against max_ptime and max_mtu */
+    if (GST_BUFFER_SIZE (buffer) >= min_payload_len &&
+        GST_BUFFER_SIZE (buffer) <= max_payload_len) {
+      ret = gst_base_rtp_audio_payload_push (basertpaudiopayload,
+          GST_BUFFER_DATA (buffer), GST_BUFFER_SIZE (buffer),
+          GST_BUFFER_TIMESTAMP (buffer));
+      gst_buffer_unref (buffer);
+
+      return ret;
+    }
+
+    available = GST_BUFFER_SIZE (buffer);
+    data = (guint8 *) GST_BUFFER_DATA (buffer);
+  }
+
+  while (available >= min_payload_len) {
+    gfloat num, datarate;
+
+    payload_len =
+        MIN (max_payload_len, (available / sample_size) * sample_size);
+
+    if (use_adapter) {
+      data = gst_adapter_peek (basertpaudiopayload->priv->adapter, payload_len);
+    }
+
+    ret =
+        gst_base_rtp_audio_payload_push (basertpaudiopayload, data, payload_len,
         basertpaudiopayload->base_ts);
 
-    gfloat num = payload_len;
-    gfloat datarate = (sample_size * basepayload->clock_rate);
+    num = payload_len;
+    datarate = (sample_size * basepayload->clock_rate);
 
     basertpaudiopayload->base_ts +=
         /* payload_len (bytes) * nsecs/sec / datarate (bytes*sec) */
-        num / datarate * GST_SECOND;
+        gst_gdouble_to_guint64 (num / datarate * GST_SECOND);
     GST_DEBUG_OBJECT (basertpaudiopayload, "New ts is %" GST_TIME_FORMAT,
         GST_TIME_ARGS (basertpaudiopayload->base_ts));
 
-    available -= payload_len;
-    data += payload_len;
+    if (use_adapter) {
+      gst_adapter_flush (basertpaudiopayload->priv->adapter, payload_len);
+      available = gst_adapter_available (basertpaudiopayload->priv->adapter);
+    } else {
+      available -= payload_len;
+      data += payload_len;
+    }
   }
 
-  gst_buffer_unref (buffer);
+  if (!use_adapter) {
+    if (available != 0 && basertpaudiopayload->priv->adapter) {
+      GstBuffer *buf;
+
+      buf = gst_buffer_create_sub (buffer,
+          GST_BUFFER_SIZE (buffer) - available, available);
+      gst_adapter_push (basertpaudiopayload->priv->adapter, buf);
+    } else {
+      gst_buffer_unref (buffer);
+    }
+  }
 
   return ret;
 }
 
-static GstFlowReturn
-gst_base_rtp_audio_payload_push (GstBaseRTPPayload * basepayload, guint8 * data,
-    guint payload_len, GstClockTime timestamp)
+/**
+ * gst_base_rtp_audio_payload_push:
+ * @basepayload: a #GstBaseRTPPayload
+ * @data: data to set as payload
+ * @payload_len: length of payload
+ * @timestamp: a #GstClockTime
+ *
+ * Create an RTP buffer and store @payload_len bytes of @data as the
+ * payload. Set the timestamp on the new buffer to @timestamp before pushing
+ * the buffer downstream.
+ *
+ * Returns: a #GstFlowReturn
+ *
+ * Since: 0.10.13
+ */
+GstFlowReturn
+gst_base_rtp_audio_payload_push (GstBaseRTPAudioPayload * baseaudiopayload,
+    const guint8 * data, guint payload_len, GstClockTime timestamp)
 {
+  GstBaseRTPPayload *basepayload;
   GstBuffer *outbuf;
   guint8 *payload;
   GstFlowReturn ret;
 
-  GST_DEBUG_OBJECT (basepayload, "Pushing %d bytes ts %" GST_TIME_FORMAT,
+  basepayload = GST_BASE_RTP_PAYLOAD (baseaudiopayload);
+
+  GST_DEBUG_OBJECT (baseaudiopayload, "Pushing %d bytes ts %" GST_TIME_FORMAT,
       payload_len, GST_TIME_ARGS (timestamp));
 
   /* create buffer to hold the payload */
@@ -470,3 +592,77 @@ gst_base_rtp_audio_payload_push (GstBaseRTPPayload * basepayload, guint8 * data,
 
   return ret;
 }
+
+static GstStateChangeReturn
+gst_base_rtp_payload_audio_change_state (GstElement * element,
+    GstStateChange transition)
+{
+  GstBaseRTPAudioPayload *basertppayload;
+  GstStateChangeReturn ret;
+
+  basertppayload = GST_BASE_RTP_AUDIO_PAYLOAD (element);
+
+  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+
+  switch (transition) {
+    case GST_STATE_CHANGE_PAUSED_TO_READY:
+      if (basertppayload->priv->adapter) {
+        gst_adapter_clear (basertppayload->priv->adapter);
+      }
+      break;
+    default:
+      break;
+  }
+
+  return ret;
+}
+
+static gboolean
+gst_base_rtp_payload_audio_handle_event (GstPad * pad, GstEvent * event)
+{
+  GstBaseRTPAudioPayload *basertpaudiopayload;
+  gboolean res = TRUE;
+
+  basertpaudiopayload = GST_BASE_RTP_AUDIO_PAYLOAD (gst_pad_get_parent (pad));
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_EOS:
+      if (basertpaudiopayload->priv->adapter) {
+        gst_adapter_clear (basertpaudiopayload->priv->adapter);
+      }
+      break;
+    case GST_EVENT_FLUSH_STOP:
+      if (basertpaudiopayload->priv->adapter) {
+        gst_adapter_clear (basertpaudiopayload->priv->adapter);
+      }
+      break;
+    default:
+      break;
+  }
+
+  gst_object_unref (basertpaudiopayload);
+
+  return res;
+}
+
+/**
+ * gst_base_rtp_audio_payload_get_adapter:
+ * @basertpaudiopayload: a #GstBaseRTPAudioPayload
+ *
+ * Gets the internal adapter used by the depayloader.
+ *
+ * Returns: a #GstAdapter.
+ *
+ * Since: 0.10.13
+ */
+GstAdapter *
+gst_base_rtp_audio_payload_get_adapter (GstBaseRTPAudioPayload
+    * basertpaudiopayload)
+{
+  GstAdapter *adapter;
+
+  if ((adapter = basertpaudiopayload->priv->adapter))
+    g_object_ref (adapter);
+
+  return adapter;
+}
