@@ -42,8 +42,6 @@ G_BEGIN_DECLS
  * @GST_RTCP_TYPE_SDES: Source description
  * @GST_RTCP_TYPE_BYE: Goodbye
  * @GST_RTCP_TYPE_APP: Application defined
- * @GST_RTCP_TYPE_RTPFB: Transport layer feedback. Since: 0.10.23
- * @GST_RTCP_TYPE_PSFB: Payload-specific feedback. Since: 0.10.23
  *
  * Different RTCP packet types.
  */
@@ -54,52 +52,9 @@ typedef enum
   GST_RTCP_TYPE_RR      = 201,
   GST_RTCP_TYPE_SDES    = 202,
   GST_RTCP_TYPE_BYE     = 203,
-  GST_RTCP_TYPE_APP     = 204,
-  GST_RTCP_TYPE_RTPFB   = 205,
-  GST_RTCP_TYPE_PSFB    = 206
+  GST_RTCP_TYPE_APP     = 204
 } GstRTCPType;
 
-/**
- * GstRTCPFBType:
- * @GST_RTCP_FB_TYPE_INVALID: Invalid type
- * @GST_RTCP_RTPFB_TYPE_NACK: Generic NACK
- * @GST_RTCP_RTPFB_TYPE_TMMBR: Temporary Maximum Media Stream Bit Rate Request
- * @GST_RTCP_RTPFB_TYPE_TMMBN: Temporary Maximum Media Stream Bit Rate
- *    Notification
- * @GST_RTCP_PSFB_TYPE_PLI: Picture Loss Indication
- * @GST_RTCP_PSFB_TYPE_SLI: Slice Loss Indication
- * @GST_RTCP_PSFB_TYPE_RPSI: Reference Picture Selection Indication
- * @GST_RTCP_PSFB_TYPE_AFB: Application layer Feedback
- * @GST_RTCP_PSFB_TYPE_FIR: Full Intra Request Command
- * @GST_RTCP_PSFB_TYPE_TSTR: Temporal-Spatial Trade-off Request
- * @GST_RTCP_PSFB_TYPE_TSTN: Temporal-Spatial Trade-off Notification
- * @GST_RTCP_PSFB_TYPE_VBCN: Video Back Channel Message
- *
- * Different types of feedback messages.
- *
- * Since: 0.10.23
- */
-typedef enum
-{
-  /* generic */
-  GST_RTCP_FB_TYPE_INVALID    = 0,
-  /* RTPFB types */
-  GST_RTCP_RTPFB_TYPE_NACK    = 1,
-  /* RTPFB types assigned in RFC 5104 */
-  GST_RTCP_RTPFB_TYPE_TMMBR   = 3,
-  GST_RTCP_RTPFB_TYPE_TMMBN   = 4,
-  /* PSFB types */
-  GST_RTCP_PSFB_TYPE_PLI      = 1,
-  GST_RTCP_PSFB_TYPE_SLI      = 2,
-  GST_RTCP_PSFB_TYPE_RPSI     = 3,
-  GST_RTCP_PSFB_TYPE_AFB      = 15,
-  /* PSFB types assigned in RFC 5104 */
-  GST_RTCP_PSFB_TYPE_FIR      = 4,
-  GST_RTCP_PSFB_TYPE_TSTR     = 5,
-  GST_RTCP_PSFB_TYPE_TSTN     = 6,
-  GST_RTCP_PSFB_TYPE_VBCN     = 7,
-} GstRTCPFBType;
-
 /** 
  * GstRTCPSDESType:
  * @GST_RTCP_SDES_INVALID: Invalid SDES entry
@@ -214,7 +169,7 @@ gboolean        gst_rtcp_packet_move_to_next      (GstRTCPPacket *packet);
 
 gboolean        gst_rtcp_buffer_add_packet        (GstBuffer *buffer, GstRTCPType type,
 		                                   GstRTCPPacket *packet);
-gboolean        gst_rtcp_packet_remove            (GstRTCPPacket *packet);
+void            gst_rtcp_packet_remove            (GstRTCPPacket *packet);
 
 /* working with packets */
 gboolean        gst_rtcp_packet_get_padding       (GstRTCPPacket *packet);
@@ -224,15 +179,15 @@ guint16         gst_rtcp_packet_get_length        (GstRTCPPacket *packet);
 
 
 /* sender reports */ 
-void            gst_rtcp_packet_sr_get_sender_info    (GstRTCPPacket *packet, guint32 *ssrc, 
-                                                       guint64 *ntptime, guint32 *rtptime, 
-						       guint32 *packet_count, guint32 *octet_count);
-void            gst_rtcp_packet_sr_set_sender_info    (GstRTCPPacket *packet, guint32 ssrc, 
-                                                       guint64 ntptime, guint32 rtptime, 
-						       guint32 packet_count, guint32 octet_count);
+void            gst_rtcp_packet_sr_get_sender_info     (GstRTCPPacket *packet, guint32 *ssrc, 
+                                                        guint64 *ntptime, guint32 *rtptime, 
+						        guint32 *packet_count, guint32 *octet_count);
+void            gst_rtcp_packet_sr_set_sender_info     (GstRTCPPacket *packet, guint32 ssrc, 
+                                                        guint64 ntptime, guint32 rtptime, 
+						        guint32 packet_count, guint32 octet_count);
 /* receiver reports */ 
-guint32         gst_rtcp_packet_rr_get_ssrc           (GstRTCPPacket *packet);
-void            gst_rtcp_packet_rr_set_ssrc           (GstRTCPPacket *packet, guint32 ssrc);
+guint32         gst_rtcp_packet_rr_get_ssrc            (GstRTCPPacket *packet);
+void            gst_rtcp_packet_rr_set_ssrc            (GstRTCPPacket *packet, guint32 ssrc);
 
 
 /* report blocks for SR and RR */
@@ -260,40 +215,19 @@ gboolean        gst_rtcp_packet_sdes_next_entry       (GstRTCPPacket *packet);
 gboolean        gst_rtcp_packet_sdes_get_entry        (GstRTCPPacket *packet, 
                                                        GstRTCPSDESType *type, guint8 *len,
 						       guint8 **data);
-gboolean        gst_rtcp_packet_sdes_copy_entry       (GstRTCPPacket *packet, 
-                                                       GstRTCPSDESType *type, guint8 *len,
-						       guint8 **data);
 
 gboolean        gst_rtcp_packet_sdes_add_item         (GstRTCPPacket *packet, guint32 ssrc);
 gboolean        gst_rtcp_packet_sdes_add_entry        (GstRTCPPacket *packet, GstRTCPSDESType type, 
                                                        guint8 len, const guint8 *data);
 
 /* bye packet */
-guint           gst_rtcp_packet_bye_get_ssrc_count    (GstRTCPPacket *packet);
-guint32         gst_rtcp_packet_bye_get_nth_ssrc      (GstRTCPPacket *packet, guint nth);
-gboolean        gst_rtcp_packet_bye_add_ssrc          (GstRTCPPacket *packet, guint32 ssrc);
-gboolean        gst_rtcp_packet_bye_add_ssrcs         (GstRTCPPacket *packet, guint32 *ssrc, guint len);
-guint8          gst_rtcp_packet_bye_get_reason_len    (GstRTCPPacket *packet);
-gchar*          gst_rtcp_packet_bye_get_reason        (GstRTCPPacket *packet);
-gboolean        gst_rtcp_packet_bye_set_reason        (GstRTCPPacket *packet, const gchar *reason);
-
-/* feedback packets */
-guint32         gst_rtcp_packet_fb_get_sender_ssrc    (GstRTCPPacket *packet);
-void            gst_rtcp_packet_fb_set_sender_ssrc    (GstRTCPPacket *packet, guint32 ssrc);
-guint32         gst_rtcp_packet_fb_get_media_ssrc     (GstRTCPPacket *packet);
-void            gst_rtcp_packet_fb_set_media_ssrc     (GstRTCPPacket *packet, guint32 ssrc);
-GstRTCPFBType   gst_rtcp_packet_fb_get_type           (GstRTCPPacket *packet);
-void            gst_rtcp_packet_fb_set_type           (GstRTCPPacket *packet, GstRTCPFBType type);
-guint16         gst_rtcp_packet_fb_get_fci_length     (GstRTCPPacket *packet);
-gboolean        gst_rtcp_packet_fb_set_fci_length     (GstRTCPPacket *packet, guint16 wordlen);
-guint8 *        gst_rtcp_packet_fb_get_fci            (GstRTCPPacket *packet);
-
-/* helper functions */
-guint64         gst_rtcp_ntp_to_unix                  (guint64 ntptime);
-guint64         gst_rtcp_unix_to_ntp                  (guint64 unixtime);
-
-const gchar *   gst_rtcp_sdes_type_to_name            (GstRTCPSDESType type);
-GstRTCPSDESType gst_rtcp_sdes_name_to_type            (const gchar *name);
+guint           gst_rtcp_packet_bye_get_ssrc_count     (GstRTCPPacket *packet);
+guint32         gst_rtcp_packet_bye_get_nth_ssrc       (GstRTCPPacket *packet, guint nth);
+gboolean        gst_rtcp_packet_bye_add_ssrc           (GstRTCPPacket *packet, guint32 ssrc);
+gboolean        gst_rtcp_packet_bye_add_ssrcs          (GstRTCPPacket *packet, guint32 *ssrc, guint len);
+guint8          gst_rtcp_packet_bye_get_reason_len     (GstRTCPPacket *packet);
+gchar*          gst_rtcp_packet_bye_get_reason         (GstRTCPPacket *packet);
+gboolean        gst_rtcp_packet_bye_set_reason         (GstRTCPPacket *packet, const gchar *reason);
 
 G_END_DECLS
 
