@@ -192,7 +192,7 @@ static const gchar *genres[] = {
   "Synthpop"
 };
 
-static GstTagEntryMatch tag_matches[] = {
+static const GstTagEntryMatch tag_matches[] = {
   {GST_TAG_TITLE, "TIT2"},
   {GST_TAG_ALBUM, "TALB"},
   {GST_TAG_TRACK_NUMBER, "TRCK"},
@@ -204,6 +204,10 @@ static GstTagEntryMatch tag_matches[] = {
   {GST_TAG_ALBUM_VOLUME_NUMBER, "TPOS"},
   {GST_TAG_DURATION, "TLEN"},
   {GST_TAG_ISRC, "TSRC"},
+  {GST_TAG_IMAGE, "APIC"},
+  {GST_TAG_ENCODER, "TSSE"},
+  {GST_TAG_BEATS_PER_MINUTE, "TBPM"},
+  {GST_TAG_MUSICBRAINZ_SORTNAME, "TSOP"},
   {NULL, NULL}
 };
 
@@ -235,7 +239,7 @@ gst_tag_from_id3_tag (const gchar * id3_tag)
   return NULL;
 }
 
-static GstTagEntryMatch user_tag_matches[] = {
+static const GstTagEntryMatch user_tag_matches[] = {
   /* musicbrainz identifiers being used in the real world (foobar2000) */
   {GST_TAG_MUSICBRAINZ_ARTISTID, "TXXX|musicbrainz_artistid"},
   {GST_TAG_MUSICBRAINZ_ALBUMID, "TXXX|musicbrainz_albumid"},
@@ -247,7 +251,12 @@ static GstTagEntryMatch user_tag_matches[] = {
   {GST_TAG_MUSICBRAINZ_ALBUMID, "TXXX|MusicBrainz Album Id"},
   {GST_TAG_MUSICBRAINZ_ALBUMARTISTID, "TXXX|MusicBrainz Album Artist Id"},
   {GST_TAG_MUSICBRAINZ_TRMID, "TXXX|MusicBrainz TRM Id"},
-  {NULL, NULL}
+  /* foobar2000 uses these identifiers to store gain/peak information in
+   * ID3v2 tags <= v2.3.0. In v2.4.0 there's the RVA2 frame for that */
+  {GST_TAG_TRACK_GAIN, "TXXX|replaygain_track_gain"},
+  {GST_TAG_TRACK_PEAK, "TXXX|replaygain_track_peak"},
+  {GST_TAG_ALBUM_GAIN, "TXXX|replaygain_album_gain"},
+  {GST_TAG_ALBUM_PEAK, "TXXX|replaygain_album_peak"}
 };
 
 /**
@@ -311,66 +320,14 @@ static void
 gst_tag_extract_id3v1_string (GstTagList * list, const gchar * tag,
     const gchar * start, const guint size)
 {
-  const gchar *env;
-  gsize bytes_read;
+  const gchar *env_vars[] = { "GST_ID3V1_TAG_ENCODING",
+    "GST_ID3_TAG_ENCODING", "GST_TAG_ENCODING", NULL
+  };
   gchar *utf8;
 
-  /* Should we try the charsets specified
-   * via environment variables FIRST ? */
-  if (g_utf8_validate (start, size, NULL)) {
-    utf8 = g_strndup (start, size);
-    goto beach;
-  }
+  utf8 = gst_tag_freeform_string_to_utf8 (start, size, env_vars);
 
-  env = g_getenv ("GST_ID3V1_TAG_ENCODING");
-  if (!env || *env == '\0')
-    env = g_getenv ("GST_ID3_TAG_ENCODING");
-  if (!env || *env == '\0')
-    env = g_getenv ("GST_TAG_ENCODING");
-
-  /* Try charsets specified via the environment */
-  if (env && *env != '\0') {
-    gchar **c, **csets;
-
-    csets = g_strsplit (env, G_SEARCHPATH_SEPARATOR_S, -1);
-
-    for (c = csets; c && *c; ++c) {
-      if ((utf8 =
-              g_convert (start, size, "UTF-8", *c, &bytes_read, NULL, NULL))) {
-        if (bytes_read == size) {
-          g_strfreev (csets);
-          goto beach;
-        }
-        g_free (utf8);
-        utf8 = NULL;
-      }
-    }
-  }
-  /* Try current locale (if not UTF-8) */
-  if (!g_get_charset (&env)) {
-    if ((utf8 = g_locale_to_utf8 (start, size, &bytes_read, NULL, NULL))) {
-      if (bytes_read == size) {
-        goto beach;
-      }
-      g_free (utf8);
-      utf8 = NULL;
-    }
-  }
-
-  /* Try ISO-8859-1 */
-  utf8 =
-      g_convert (start, size, "UTF-8", "ISO-8859-1", &bytes_read, NULL, NULL);
-  if (utf8 != NULL && bytes_read == size) {
-    goto beach;
-  }
-
-  g_free (utf8);
-  return;
-
-beach:
-
-  g_strchomp (utf8);
-  if (utf8 && utf8[0] != '\0') {
+  if (utf8 && *utf8 != '\0') {
     gst_tag_list_add (list, GST_TAG_MERGE_REPLACE, tag, utf8, NULL);
   }
 
