@@ -21,8 +21,11 @@
 #include "config.h"
 #endif
 
+#include <gst/gst-i18n-plugin.h>
+
 #include <string.h>
 #include <gst/gst.h>
+#include <gst/pbutils/pbutils.h>
 
 #include "gstplay-marshal.h"
 
@@ -130,7 +133,7 @@ static void gst_decode_bin_finalize (GObject * object);
 static GstStateChangeReturn gst_decode_bin_change_state (GstElement * element,
     GstStateChange transition);
 
-static void add_fakesink (GstDecodeBin * decode_bin);
+static gboolean add_fakesink (GstDecodeBin * decode_bin);
 static void remove_fakesink (GstDecodeBin * decode_bin);
 
 static void dynamic_free (GstDynamic * dyn);
@@ -234,7 +237,7 @@ gst_decode_bin_class_init (GstDecodeBinClass * klass)
 
 /* check if the bin is dynamic.
  *
- * If there are no outstanding dynamic connections, the bin is 
+ * If there are no outstanding dynamic connections, the bin is
  * considered to be non-dynamic.
  */
 static gboolean
@@ -409,7 +412,7 @@ dynamic_add (GstElement * element, GstPad * pad, GstDecodeBin * decode_bin)
   struct DynFind find_info;
   GList *found;
 
-  g_return_if_fail (element != NULL || pad != NULL);
+  g_return_if_fail (element != NULL);
 
   /* do a search that this entry doesn't already exist */
   find_info.elem = element;
@@ -421,25 +424,21 @@ dynamic_add (GstElement * element, GstPad * pad, GstDecodeBin * decode_bin)
 
   /* take refs */
   dyn = g_new0 (GstDynamic, 1);
-  dyn->element = element;
-  dyn->pad = pad;
+  dyn->element = gst_object_ref (element);
   dyn->decode_bin = gst_object_ref (decode_bin);
-  if (element) {
+  if (pad) {
+    dyn->pad = gst_object_ref (pad);
+    GST_DEBUG_OBJECT (decode_bin, "dynamic create for pad %" GST_PTR_FORMAT,
+        pad);
+    dyn->caps_sig_id = g_signal_connect (G_OBJECT (pad), "notify::caps",
+        G_CALLBACK (new_caps), dyn);
+  } else {
     GST_DEBUG_OBJECT (decode_bin, "dynamic create for element %"
         GST_PTR_FORMAT, element);
-
-    gst_object_ref (element);
     dyn->np_sig_id = g_signal_connect (G_OBJECT (element), "pad-added",
         G_CALLBACK (new_pad), dyn);
     dyn->nmp_sig_id = g_signal_connect (G_OBJECT (element), "no-more-pads",
         G_CALLBACK (no_more_pads), dyn);
-  } else {
-    GST_DEBUG_OBJECT (decode_bin, "dynamic create for pad %" GST_PTR_FORMAT,
-        pad);
-
-    gst_object_ref (pad);
-    dyn->caps_sig_id = g_signal_connect (G_OBJECT (pad), "notify::caps",
-        G_CALLBACK (new_caps), dyn);
   }
 
   /* and add this element to the dynamic elements */
@@ -497,7 +496,7 @@ free_dynamics (GstDecodeBin * decode_bin)
 }
 
 /* this function runs through the element factories and returns a list
- * of all elements that are able to sink the given caps 
+ * of all elements that are able to sink the given caps
  */
 static GList *
 find_compatibles (GstDecodeBin * decode_bin, const GstCaps * caps)
@@ -569,11 +568,11 @@ free_pad_probes (GstDecodeBin * decode_bin)
   decode_bin->probes = NULL;
 }
 
-static void
+static gboolean
 add_fakesink (GstDecodeBin * decode_bin)
 {
   if (decode_bin->fakesink != NULL)
-    return;
+    return TRUE;
 
   g_mutex_lock (decode_bin->cb_mutex);
 
@@ -591,14 +590,14 @@ add_fakesink (GstDecodeBin * decode_bin)
     decode_bin->fakesink = NULL;
   }
   g_mutex_unlock (decode_bin->cb_mutex);
-  return;
+  return TRUE;
 
   /* ERRORS */
 no_fakesink:
   {
     g_warning ("can't find fakesink element, decodebin will not work");
     g_mutex_unlock (decode_bin->cb_mutex);
-    return;
+    return FALSE;
   }
 }
 
@@ -625,9 +624,6 @@ remove_fakesink (GstDecodeBin * decode_bin)
 
   if (removed_fakesink) {
     free_pad_probes (decode_bin);
-
-    gst_element_post_message (GST_ELEMENT_CAST (decode_bin),
-        gst_message_new_state_dirty (GST_OBJECT_CAST (decode_bin)));
   }
 }
 
@@ -674,7 +670,7 @@ pad_probe (GstPad * pad, GstMiniObject * data, GstDecodeBin * decode_bin)
  * If the pad has a raw format, this function will create a ghostpad
  * for the pad onto the decodebin.
  *
- * If no compatible elements could be found, this function will signal 
+ * If no compatible elements could be found, this function will signal
  * the unknown_type signal.
  */
 static void
@@ -779,6 +775,9 @@ unknown_type:
     GST_LOG_OBJECT (pad, "unknown type found, fire signal");
     g_signal_emit (G_OBJECT (decode_bin),
         gst_decode_bin_signals[SIGNAL_UNKNOWN_TYPE], 0, pad, caps);
+
+    gst_element_post_message (GST_ELEMENT_CAST (decode_bin),
+        gst_missing_decoder_message_new (GST_ELEMENT_CAST (decode_bin), caps));
     return;
   }
 dont_know_yet:
@@ -794,12 +793,12 @@ many_types:
 setup_caps_delay:
   {
     GST_LOG_OBJECT (pad, "setting up a delayed link");
-    dynamic_add (NULL, pad, decode_bin);
+    dynamic_add (element, pad, decode_bin);
     return;
   }
 }
 
-/* Decide whether an element is a demuxer based on the 
+/* Decide whether an element is a demuxer based on the
  * klass and number/type of src pad templates it has */
 static gboolean
 is_demuxer_element (GstElement * srcelement)
@@ -814,7 +813,7 @@ is_demuxer_element (GstElement * srcelement)
   klass = gst_element_factory_get_klass (srcfactory);
 
   /* Can't be a demuxer unless it has Demux in the klass name */
-  if (!strstr (klass, "Demux"))
+  if (klass == NULL || !strstr (klass, "Demux"))
     return FALSE;
 
   /* Walk the src pad templates and count how many the element
@@ -912,7 +911,7 @@ try_to_link_1 (GstDecodeBin * decode_bin, GstElement * srcelement, GstPad * pad,
     }
 
     /* try to link the given pad to a sinkpad */
-    /* FIXME, find the sinkpad by looping over the pads instead of 
+    /* FIXME, find the sinkpad by looping over the pads instead of
      * looking it up by name */
     if ((sinkpad = gst_element_get_pad (element, "sink")) == NULL) {
       /* if no pad is found we can't do anything */
@@ -968,7 +967,7 @@ try_to_link_1 (GstDecodeBin * decode_bin, GstElement * srcelement, GstPad * pad,
       /* get rid of the sinkpad now */
       gst_object_unref (sinkpad);
 
-      /* Set the queue to paused and set the pointer to NULL so we don't 
+      /* Set the queue to paused and set the pointer to NULL so we don't
        * remove it below */
       if (queue != NULL) {
         gst_element_set_state (queue, GST_STATE_PAUSED);
@@ -1048,7 +1047,7 @@ get_our_ghost_pad (GstDecodeBin * decode_bin, GstPad * pad)
 }
 
 /* remove all downstream elements starting from the given pad.
- * Also make sure to remove the ghostpad we created for the raw 
+ * Also make sure to remove the ghostpad we created for the raw
  * decoded stream.
  */
 static void
@@ -1070,7 +1069,7 @@ remove_element_chain (GstDecodeBin * decode_bin, GstPad * pad)
   GST_DEBUG_OBJECT (decode_bin, "%s:%s", GST_DEBUG_PAD_NAME (pad));
   int_links = gst_pad_get_internal_links (pad);
 
-  /* remove all elements linked to this pad up to the ghostpad 
+  /* remove all elements linked to this pad up to the ghostpad
    * that we created for this stream */
   for (walk = int_links; walk; walk = g_list_next (walk)) {
     GstPad *pad;
@@ -1134,7 +1133,7 @@ remove_element_chain (GstDecodeBin * decode_bin, GstPad * pad)
   gst_bin_remove (GST_BIN (decode_bin), elem);
 }
 
-/* there are @bytes bytes in @queue, enlarge it 
+/* there are @bytes bytes in @queue, enlarge it
  *
  * Returns: new max number of bytes in @queue
  */
@@ -1167,13 +1166,14 @@ queue_underrun_cb (GstElement * queue, GstDecodeBin * decode_bin)
 {
   /* FIXME: we don't really do anything here for now. Ideally we should
    * see if some of the queues are filled and increase their values
-   * in that case. 
+   * in that case.
    * Note: be very carefull with thread safety here as this underrun
    * signal is done from the streaming thread of queue srcpad which
    * is different from the pad_added (where we add the queue to the
    * list) and the overrun signals that are signalled from the
    * demuxer thread.
    */
+  GST_DEBUG_OBJECT (decode_bin, "got underrun");
 }
 
 /* Make sure we don't have a full queue and empty queue situation */
@@ -1311,7 +1311,7 @@ no_more_pads (GstElement * element, GstDynamic * dynamic)
 static void
 new_caps (GstPad * pad, GParamSpec * unused, GstDynamic * dynamic)
 {
-  GST_DEBUG_OBJECT (dynamic->decode_bin, "delayed link triggered\n");
+  GST_DEBUG_OBJECT (dynamic->decode_bin, "delayed link triggered");
 
   new_pad (dynamic->element, pad, dynamic);
 
@@ -1497,7 +1497,7 @@ close_link (GstElement * element, GstDecodeBin * decode_bin)
   }
 
   /* Check if this is an element with more than 1 pad. If this element
-   * has more than 1 pad, we need to be carefull not to signal the 
+   * has more than 1 pad, we need to be carefull not to signal the
    * no_more_pads signal after connecting the first pad. */
   more = g_list_length (to_connect) > 1;
 
@@ -1541,6 +1541,19 @@ type_found (GstElement * typefind, guint probability, GstCaps * caps,
 
   GST_DEBUG_OBJECT (decode_bin, "typefind found caps %" GST_PTR_FORMAT, caps);
 
+  /* special-case text/plain: we only want to accept it as a raw type if it
+   * comes from a subtitel parser element or a demuxer, but not if it is the
+   * type of the entire stream, in which case we just want to error out */
+  if (typefind == decode_bin->typefind &&
+      gst_structure_has_name (gst_caps_get_structure (caps, 0), "text/plain")) {
+    gst_element_no_more_pads (GST_ELEMENT (decode_bin));
+    /* we can't handle this type of stream */
+    GST_ELEMENT_ERROR (decode_bin, STREAM, WRONG_TYPE,
+        (_("This appears to be a text file")),
+        ("decodebin cannot decode plain text files"));
+    goto shutting_down;
+  }
+
   /* autoplug the new pad with the caps that the signal gave us. */
   pad = gst_element_get_pad (typefind, "src");
   close_pad_link (typefind, pad, caps, decode_bin, FALSE);
@@ -1655,14 +1668,16 @@ gst_decode_bin_change_state (GstElement * element, GstStateChange transition)
       decode_bin->numpads = 0;
       decode_bin->numwaiting = 0;
       decode_bin->dynamics = NULL;
+      if (decode_bin->typefind == NULL)
+        goto missing_typefind;
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       GST_OBJECT_LOCK (decode_bin);
       decode_bin->shutting_down = FALSE;
       GST_OBJECT_UNLOCK (decode_bin);
 
-      add_fakesink (decode_bin);
-      break;
+      if (!add_fakesink (decode_bin))
+        goto missing_fakesink;
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
@@ -1677,6 +1692,8 @@ gst_decode_bin_change_state (GstElement * element, GstStateChange transition)
   }
 
   ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+  if (ret == GST_STATE_CHANGE_FAILURE)
+    return ret;
 
   switch (transition) {
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
@@ -1692,6 +1709,21 @@ gst_decode_bin_change_state (GstElement * element, GstStateChange transition)
   }
 
   return ret;
+/* ERRORS */
+missing_typefind:
+  {
+    gst_element_post_message (element,
+        gst_missing_element_message_new (element, "typefind"));
+    GST_ELEMENT_ERROR (element, CORE, MISSING_PLUGIN, (NULL), ("no typefind!"));
+    return GST_STATE_CHANGE_FAILURE;
+  }
+missing_fakesink:
+  {
+    gst_element_post_message (element,
+        gst_missing_element_message_new (element, "fakesink"));
+    GST_ELEMENT_ERROR (element, CORE, MISSING_PLUGIN, (NULL), ("no fakesink!"));
+    return GST_STATE_CHANGE_FAILURE;
+  }
 }
 
 static gboolean
@@ -1699,6 +1731,12 @@ plugin_init (GstPlugin * plugin)
 {
   GST_DEBUG_CATEGORY_INIT (gst_decode_bin_debug, "decodebin", 0, "decoder bin");
 
+#ifdef ENABLE_NLS
+  GST_DEBUG ("binding text domain %s to locale dir %s", GETTEXT_PACKAGE,
+      LOCALEDIR);
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+#endif /* ENABLE_NLS */
+
   return gst_element_register (plugin, "decodebin", GST_RANK_NONE,
       GST_TYPE_DECODE_BIN);
 }
