@@ -36,6 +36,8 @@
 #include "config.h"
 #endif
 
+#include <gst/gst-i18n-plugin.h>
+
 #include <string.h>
 #include <gst/gst.h>
 #include <gst/pbutils/pbutils.h>
@@ -446,7 +448,8 @@ gst_decode_bin_factory_filter (GstPluginFeature * feature,
   klass = gst_element_factory_get_klass (GST_ELEMENT_FACTORY (feature));
   /* only demuxers, decoders and parsers can play */
   if (strstr (klass, "Demux") == NULL &&
-      strstr (klass, "Decoder") == NULL && strstr (klass, "Parse") == NULL) {
+      strstr (klass, "Decoder") == NULL && strstr (klass, "Parse") == NULL &&
+      strstr (klass, "Depayloader") == NULL) {
     return FALSE;
   }
 
@@ -835,6 +838,7 @@ connect_pad (GstDecodeBin * dbin, GstElement * src, GstPad * pad,
 {
   gboolean res = FALSE;
   GList *tmp;
+  GstPad *mqpad = NULL;
 
   g_return_val_if_fail (factories != NULL, FALSE);
   GST_DEBUG_OBJECT (dbin, "pad %s:%s , group:%p",
@@ -842,8 +846,6 @@ connect_pad (GstDecodeBin * dbin, GstElement * src, GstPad * pad,
 
   /* 1. is element demuxer or parser */
   if (is_demuxer_element (src)) {
-    GstPad *mqpad;
-
     GST_LOG_OBJECT (src, "is a demuxer, connecting the pad through multiqueue");
 
     if (!group)
@@ -908,7 +910,7 @@ connect_pad (GstDecodeBin * dbin, GstElement * src, GstPad * pad,
       gst_bin_remove (GST_BIN (dbin), element);
       continue;
     }
-
+    gst_object_unref (sinkpad);
     GST_LOG_OBJECT (dbin, "linked on pad %s:%s", GST_DEBUG_PAD_NAME (pad));
 
     /* link this element further */
@@ -920,7 +922,6 @@ connect_pad (GstDecodeBin * dbin, GstElement * src, GstPad * pad,
       GST_WARNING_OBJECT (dbin, "Couldn't set %s to PAUSED",
           GST_ELEMENT_NAME (element));
       gst_element_set_state (element, GST_STATE_NULL);
-      gst_object_unref (sinkpad);
       gst_bin_remove (GST_BIN (dbin), element);
       continue;
     }
@@ -929,6 +930,9 @@ connect_pad (GstDecodeBin * dbin, GstElement * src, GstPad * pad,
     break;
   }
 
+  if (mqpad)
+    gst_object_unref (mqpad);
+
 beach:
   return res;
 }
@@ -1058,6 +1062,7 @@ expose_pad (GstDecodeBin * dbin, GstElement * src, GstPad * pad,
 {
   gboolean newgroup = FALSE;
   gboolean isdemux;
+  GstPad *mqpad = NULL;
 
   GST_DEBUG_OBJECT (dbin, "pad %s:%s, group:%p",
       GST_DEBUG_PAD_NAME (pad), group);
@@ -1074,8 +1079,6 @@ expose_pad (GstDecodeBin * dbin, GstElement * src, GstPad * pad,
   isdemux = is_demuxer_element (src);
 
   if (isdemux || newgroup) {
-    GstPad *mqpad;
-
     GST_LOG_OBJECT (src, "is a demuxer, connecting the pad through multiqueue");
 
     if (!(mqpad = gst_decode_group_control_demuxer_pad (group, pad)))
@@ -1091,6 +1094,9 @@ expose_pad (GstDecodeBin * dbin, GstElement * src, GstPad * pad,
      * group as being complete. */
     gst_decode_group_set_complete (group);
   }
+  if (mqpad)
+    gst_object_unref (mqpad);
+
 beach:
   return;
 }
@@ -1823,7 +1829,10 @@ restart:
         GstPad *peerpad = NULL;
 
         if ((peerpad = gst_pad_get_peer (pad))) {
-          GstObject *parent = gst_pad_get_parent (peerpad);
+          GstObject *parent;
+
+          parent = gst_pad_get_parent (peerpad);
+          gst_object_unref (peerpad);
 
           if (parent && GST_IS_ELEMENT (parent))
             deactivate_free_recursive (group, GST_ELEMENT (parent));
@@ -2107,6 +2116,12 @@ plugin_init (GstPlugin * plugin)
   GST_DEBUG_CATEGORY_INIT (gst_decode_bin_debug, "decodebin2", 0,
       "decoder bin");
 
+#ifdef ENABLE_NLS
+  GST_DEBUG ("binding text domain %s to locale dir %s", GETTEXT_PACKAGE,
+      LOCALEDIR);
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+#endif /* ENABLE_NLS */
+
   return gst_element_register (plugin, "decodebin2", GST_RANK_NONE,
       GST_TYPE_DECODE_BIN);
 }
