@@ -32,6 +32,7 @@
 #include "gstssaparse.h"
 #include "samiparse.h"
 #include "tmplayerparse.h"
+#include "mpl2parse.h"
 
 GST_DEBUG_CATEGORY (sub_parse_debug);
 
@@ -58,12 +59,12 @@ GST_ELEMENT_DETAILS ("Subtitle parser",
     "Gustavo J. A. M. Carneiro <gjc@inescporto.pt>\n"
     "Ronald S. Bultje <rbultje@ronald.bitfreak.net>");
 
-#ifndef GST_DISABLE_LOADSAVE_REGISTRY
+#ifndef GST_DISABLE_XML
 static GstStaticPadTemplate sink_templ = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("application/x-subtitle; application/x-subtitle-sami; "
-        "application/x-subtitle-tmplayer")
+        "application/x-subtitle-tmplayer; application/x-subtitle-mpl2")
     );
 #else
 static GstStaticPadTemplate sink_templ = GST_STATIC_PAD_TEMPLATE ("sink",
@@ -137,10 +138,6 @@ gst_sub_parse_dispose (GObject * object)
 
   GST_DEBUG_OBJECT (subparse, "cleaning up subtitle parser");
 
-  if (subparse->segment) {
-    gst_segment_free (subparse->segment);
-    subparse->segment = NULL;
-  }
   if (subparse->encoding) {
     g_free (subparse->encoding);
     subparse->encoding = NULL;
@@ -194,14 +191,8 @@ gst_sub_parse_init (GstSubParse * subparse)
   subparse->textbuf = g_string_new (NULL);
   subparse->parser_type = GST_SUB_PARSE_FORMAT_UNKNOWN;
   subparse->flushing = FALSE;
-  subparse->segment = gst_segment_new ();
-  if (subparse->segment) {
-    gst_segment_init (subparse->segment, GST_FORMAT_TIME);
-    subparse->need_segment = TRUE;
-  } else {
-    GST_WARNING_OBJECT (subparse, "segment creation failed");
-    g_assert_not_reached ();
-  }
+  gst_segment_init (&subparse->segment, GST_FORMAT_TIME);
+  subparse->need_segment = TRUE;
   subparse->encoding = g_strdup (DEFAULT_ENCODING);
 }
 
@@ -243,14 +234,11 @@ gst_sub_parse_src_event (GstPad * pad, GstEvent * event)
 
       if (ret) {
         /* Apply the seek to our segment */
-        gst_segment_set_seek (self->segment, rate, format, self->segment_flags,
+        gst_segment_set_seek (&self->segment, rate, format, self->segment_flags,
             start_type, start, stop_type, stop, &update);
 
-        GST_DEBUG_OBJECT (self, "segment configured from %" GST_TIME_FORMAT
-            " to %" GST_TIME_FORMAT ", position %" GST_TIME_FORMAT,
-            GST_TIME_ARGS (self->segment->start),
-            GST_TIME_ARGS (self->segment->stop),
-            GST_TIME_ARGS (self->segment->last_stop));
+        GST_DEBUG_OBJECT (self, "segment after seek: %" GST_SEGMENT_FORMAT,
+            &self->segment);
 
         self->next_offset = 0;
 
@@ -396,10 +384,6 @@ parse_mdvdsub (ParserState * state, const gchar * line)
   guint start_frame, end_frame;
   gint64 clip_start = 0, clip_stop = 0;
   gboolean in_seg = FALSE;
-
-  /* FIXME: hardcoded for now, but detecting the correct value is
-   * not going to be easy, I suspect... */
-  const double frames_per_sec = 24000 / 1001.;
   GString *markup;
   gchar *ret;
 
@@ -414,8 +398,32 @@ parse_mdvdsub (ParserState * state, const gchar * line)
     return NULL;
   }
 
-  state->start_time = (start_frame - 1000) / frames_per_sec * GST_SECOND;
-  state->duration = (end_frame - start_frame) / frames_per_sec * GST_SECOND;
+  /* skip the {%u}{%u} part */
+  line = strchr (line, '}') + 1;
+  line = strchr (line, '}') + 1;
+
+  /* see if there's a first line with a framerate */
+  if (state->fps == 0.0 && start_frame == 1 && end_frame == 1) {
+    gchar *rest, *end = NULL;
+
+    rest = g_strdup (line);
+    g_strdelimit (rest, ",", '.');
+    state->fps = g_ascii_strtod (rest, &end);
+    if (end == rest)
+      state->fps = 0.0;
+    GST_INFO ("framerate from file: %f ('%s')", state->fps, rest);
+    g_free (rest);
+    return NULL;
+  }
+
+  if (state->fps == 0.0) {
+    /* FIXME: hardcoded for now, is there a better way/assumption? */
+    state->fps = 24000.0 / 1001.0;
+    GST_INFO ("no framerate specified, assuming %f", state->fps);
+  }
+
+  state->start_time = start_frame / state->fps * GST_SECOND;
+  state->duration = (end_frame - start_frame) / state->fps * GST_SECOND;
 
   /* Check our segment start/stop */
   in_seg = gst_segment_clip (state->segment, GST_FORMAT_TIME,
@@ -430,10 +438,6 @@ parse_mdvdsub (ParserState * state, const gchar * line)
     return NULL;
   }
 
-  /* skip the {%u}{%u} part */
-  line = strchr (line, '}') + 1;
-  line = strchr (line, '}') + 1;
-
   markup = g_string_new (NULL);
   while (1) {
     italic = FALSE;
@@ -531,6 +535,42 @@ subrip_unescape_formatting (gchar * txt)
   }
 }
 
+
+static gboolean
+subrip_remove_unhandled_tag (gchar * start, gchar * stop)
+{
+  gchar *tag, saved;
+
+  tag = start + strlen ("&lt;");
+  if (*tag == '/')
+    ++tag;
+
+  if (g_ascii_tolower (*tag) < 'a' || g_ascii_tolower (*tag) > 'z')
+    return FALSE;
+
+  saved = *stop;
+  *stop = '\0';
+  GST_LOG ("removing unhandled tag '%s'", start);
+  *stop = saved;
+  g_memmove (start, stop, strlen (stop) + 1);
+  return TRUE;
+}
+
+/* remove tags we haven't explicitly allowed earlier on, like font tags
+ * for example */
+static void
+subrip_remove_unhandled_tags (gchar * txt)
+{
+  gchar *pos, *gt;
+
+  for (pos = txt; pos != NULL && *pos != '\0'; ++pos) {
+    if (strncmp (pos, "&lt;", 4) == 0 && (gt = strstr (pos + 4, "&gt;"))) {
+      if (subrip_remove_unhandled_tag (pos, gt + strlen ("&gt;")))
+        --pos;
+    }
+  }
+}
+
 /* we only allow <i>, <u> and <b>, so let's take a simple approach. This code
  * assumes the input has been escaped and subrip_unescape_formatting() has then
  * been run over the input! This function adds missing closing markup tags and
@@ -654,6 +694,7 @@ parse_subrip (ParserState * state, const gchar * line)
         g_string_truncate (state->buf, 0);
         state->state = 0;
         subrip_unescape_formatting (ret);
+        subrip_remove_unhandled_tags (ret);
         strip_trailing_newlines (ret);
         subrip_fix_up_markup (&ret);
         return ret;
@@ -801,7 +842,10 @@ gst_sub_parse_data_format_autodetect (gchar * match_str)
     GST_LOG ("TMPlayer (time based) format detected");
     return GST_SUB_PARSE_FORMAT_TMPLAYER;
   }
-
+  if (sscanf (match_str, "[%u][%u]", &n1, &n2) == 2) {
+    GST_LOG ("MPL2 (time based) format detected");
+    return GST_SUB_PARSE_FORMAT_MPL2;
+  }
   GST_DEBUG ("no subtitle format detected");
   return GST_SUB_PARSE_FORMAT_UNKNOWN;
 }
@@ -841,6 +885,9 @@ gst_sub_parse_format_autodetect (GstSubParse * self)
     case GST_SUB_PARSE_FORMAT_TMPLAYER:
       self->parse_line = parse_tmplayer;
       return gst_caps_new_simple ("text/plain", NULL);
+    case GST_SUB_PARSE_FORMAT_MPL2:
+      self->parse_line = parse_mpl2;
+      return gst_caps_new_simple ("text/x-pango-markup", NULL);
     case GST_SUB_PARSE_FORMAT_UNKNOWN:
     default:
       GST_DEBUG ("no subtitle format detected");
@@ -891,7 +938,7 @@ handle_buffer (GstSubParse * self, GstBuffer * buf)
 
   while ((line = get_next_line (self)) && !self->flushing) {
     /* Set segment on our parser state machine */
-    self->state.segment = self->segment;
+    self->state.segment = &self->segment;
     /* Now parse the line, out of segment lines will just return NULL */
     GST_LOG_OBJECT (self, "Parsing line '%s'", line);
     subtitle = self->parse_line (&self->state, line);
@@ -912,7 +959,7 @@ handle_buffer (GstSubParse * self, GstBuffer * buf)
         GST_BUFFER_TIMESTAMP (buf) = self->state.start_time;
         GST_BUFFER_DURATION (buf) = self->state.duration;
 
-        gst_segment_set_last_stop (self->segment, GST_FORMAT_TIME,
+        gst_segment_set_last_stop (&self->segment, GST_FORMAT_TIME,
             self->state.start_time);
 
         GST_DEBUG_OBJECT (self, "Sending text '%s', %" GST_TIME_FORMAT " + %"
@@ -925,8 +972,10 @@ handle_buffer (GstSubParse * self, GstBuffer * buf)
       g_free (subtitle);
       subtitle = NULL;
 
-      if (GST_FLOW_IS_FATAL (ret))
+      if (ret != GST_FLOW_OK) {
+        GST_DEBUG_OBJECT (self, "flow: %s", gst_flow_get_name (ret));
         break;
+      }
     }
   }
 
@@ -943,10 +992,12 @@ gst_sub_parse_chain (GstPad * sinkpad, GstBuffer * buf)
 
   /* Push newsegment if needed */
   if (self->need_segment) {
+    GST_LOG_OBJECT (self, "pushing newsegment event with %" GST_SEGMENT_FORMAT,
+        &self->segment);
+
     gst_pad_push_event (self->srcpad, gst_event_new_new_segment (FALSE,
-            self->segment->rate, self->segment->format,
-            self->segment->last_stop, self->segment->stop,
-            self->segment->time));
+            self->segment.rate, self->segment.format,
+            self->segment.last_stop, self->segment.stop, self->segment.time));
     self->need_segment = FALSE;
   }
 
@@ -967,7 +1018,8 @@ gst_sub_parse_sink_event (GstPad * pad, GstEvent * event)
     case GST_EVENT_EOS:{
       /* Make sure the last subrip chunk is pushed out even
        * if the file does not have an empty line at the end */
-      if (self->parser_type == GST_SUB_PARSE_FORMAT_SUBRIP) {
+      if (self->parser_type == GST_SUB_PARSE_FORMAT_SUBRIP ||
+          self->parser_type == GST_SUB_PARSE_FORMAT_MPL2) {
         GstBuffer *buf = gst_buffer_new_and_alloc (1 + 1);
 
         GST_DEBUG ("EOS. Pushing remaining text (if any)");
@@ -987,14 +1039,24 @@ gst_sub_parse_sink_event (GstPad * pad, GstEvent * event)
       gint64 start, stop, time;
       gboolean update;
 
-      GST_DEBUG_OBJECT (self, "received new segment");
-
       gst_event_parse_new_segment (event, &update, &rate, &format, &start,
           &stop, &time);
 
-      /* now copy over the values */
-      gst_segment_set_newsegment (self->segment, update, rate, format,
-          start, stop, time);
+      GST_DEBUG_OBJECT (self, "newsegment (%s)", gst_format_get_name (format));
+
+      if (format == GST_FORMAT_TIME) {
+        gst_segment_set_newsegment (&self->segment, update, rate, format,
+            start, stop, time);
+      } else {
+        /* if not time format, we'll either start with a 0 timestamp anyway or
+         * it's following a seek in which case we'll have saved the requested
+         * seek segment and don't want to overwrite it (remember that on a seek
+         * we always just seek back to the start in BYTES format and just throw
+         * away all text that's before the requested position; if the subtitles
+         * come from an upstream demuxer, it won't be able to handle our BYTES
+         * seek request and instead send us a newsegment from the seek request
+         * it received via its video pads instead, so all is fine then too) */
+      }
 
       ret = TRUE;
       gst_event_unref (event);
@@ -1066,6 +1128,8 @@ gst_sub_parse_change_state (GstElement * element, GstStateChange transition)
 
 /* FIXME 0.11: these caps are ugly, use app/x-subtitle + type field or so;
  * also, give different  subtitle formats really different types */
+static GstStaticCaps mpl2_caps =
+GST_STATIC_CAPS ("application/x-subtitle-mpl2");
 static GstStaticCaps tmp_caps =
 GST_STATIC_CAPS ("application/x-subtitle-tmplayer");
 static GstStaticCaps smi_caps = GST_STATIC_CAPS ("application/x-subtitle-sami");
@@ -1074,6 +1138,7 @@ static GstStaticCaps sub_caps = GST_STATIC_CAPS ("application/x-subtitle");
 #define SUB_CAPS (gst_static_caps_get (&sub_caps))
 #define SAMI_CAPS (gst_static_caps_get (&smi_caps))
 #define TMP_CAPS (gst_static_caps_get (&tmp_caps))
+#define MPL2_CAPS (gst_static_caps_get (&mpl2_caps))
 
 static void
 gst_subparse_type_find (GstTypeFind * tf, gpointer private)
@@ -1112,6 +1177,14 @@ gst_subparse_type_find (GstTypeFind * tf, gpointer private)
       GST_DEBUG ("TMPlayer (time based) format detected");
       caps = TMP_CAPS;
       break;
+      /* FIXME: our MPL2 typefinding is not really good enough to warrant
+       * returning a high probability (however, since we registered our
+       * typefinder here with a rank of MARGINAL we should pretty much only
+       * be called if most other typefinders have already run */
+    case GST_SUB_PARSE_FORMAT_MPL2:
+      GST_DEBUG ("MPL2 (time based) format detected");
+      caps = MPL2_CAPS;
+      break;
     default:
     case GST_SUB_PARSE_FORMAT_UNKNOWN:
       GST_DEBUG ("no subtitle format detected");
