@@ -530,8 +530,9 @@ gst_multi_fd_sink_class_init (GstMultiFdSinkClass * klass)
    *     values that represent respectively: total number of bytes sent, time
    *     when the client was added, time when the client was
    *     disconnected/removed, time the client is/was active, last activity
-   *     time, number of buffers dropped.
+   *     time (in epoch seconds), number of buffers dropped.
    *     All times are expressed in nanoseconds (GstClockTime).
+   *     The array can be 0-length if the client was not found.
    */
   gst_multi_fd_sink_signals[SIGNAL_GET_STATS] =
       g_signal_new ("get-stats", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
@@ -826,14 +827,14 @@ restart:
   CLIENTS_UNLOCK (sink);
 }
 
-/* "get-stats" signal implemntation
+/* "get-stats" signal implementation
  * the array returned contains:
  *
  * guint64 : bytes_sent
- * guint64 : connect time (in nanoseconds)
- * guint64 : disconnect time (in nanoseconds)
+ * guint64 : connect time (in nanoseconds, since Epoch)
+ * guint64 : disconnect time (in nanoseconds, since Epoch)
  * guint64 : time the client is/was connected (in nanoseconds)
- * guint64 : last activity time (in nanoseconds)
+ * guint64 : last activity time (in nanoseconds, since Epoch)
  * guint64 : buffers dropped due to recovery
  */
 GValueArray *
@@ -1158,7 +1159,7 @@ gst_multi_fd_sink_client_queue_buffer (GstMultiFdSink * sink,
    * buffers (because it's a new client, or because they changed) */
   caps = gst_buffer_get_caps (buffer);  /* cleaned up after streamheader */
   if (!client->caps) {
-    GST_LOG_OBJECT (sink,
+    GST_DEBUG_OBJECT (sink,
         "[fd %5d] no previous caps for this client, send streamheader",
         client->fd.fd);
     send_streamheader = TRUE;
@@ -1172,7 +1173,7 @@ gst_multi_fd_sink_client_queue_buffer (GstMultiFdSink * sink,
       s = gst_caps_get_structure (caps, 0);
       if (!gst_structure_has_field (s, "streamheader")) {
         /* no new streamheader, so nothing new to send */
-        GST_LOG_OBJECT (sink,
+        GST_DEBUG_OBJECT (sink,
             "[fd %5d] new caps do not have streamheader, not sending",
             client->fd.fd);
       } else {
@@ -1180,7 +1181,7 @@ gst_multi_fd_sink_client_queue_buffer (GstMultiFdSink * sink,
         s = gst_caps_get_structure (client->caps, 0);
         if (!gst_structure_has_field (s, "streamheader")) {
           /* no previous streamheader, so send the new one */
-          GST_LOG_OBJECT (sink,
+          GST_DEBUG_OBJECT (sink,
               "[fd %5d] previous caps did not have streamheader, sending",
               client->fd.fd);
           send_streamheader = TRUE;
@@ -1190,7 +1191,7 @@ gst_multi_fd_sink_client_queue_buffer (GstMultiFdSink * sink,
           s = gst_caps_get_structure (caps, 0);
           sh2 = gst_structure_get_value (s, "streamheader");
           if (gst_value_compare (sh1, sh2) != GST_VALUE_EQUAL) {
-            GST_LOG_OBJECT (sink,
+            GST_DEBUG_OBJECT (sink,
                 "[fd %5d] new streamheader different from old, sending",
                 client->fd.fd);
             send_streamheader = TRUE;
@@ -1213,7 +1214,7 @@ gst_multi_fd_sink_client_queue_buffer (GstMultiFdSink * sink,
         client->fd.fd, caps);
     s = gst_caps_get_structure (caps, 0);
     if (!gst_structure_has_field (s, "streamheader")) {
-      GST_LOG_OBJECT (sink,
+      GST_DEBUG_OBJECT (sink,
           "[fd %5d] no new streamheader, so nothing to send", client->fd.fd);
     } else {
       GST_LOG_OBJECT (sink,
@@ -1222,6 +1223,7 @@ gst_multi_fd_sink_client_queue_buffer (GstMultiFdSink * sink,
       sh = gst_structure_get_value (s, "streamheader");
       g_assert (G_VALUE_TYPE (sh) == GST_TYPE_ARRAY);
       buffers = g_value_peek_pointer (sh);
+      GST_DEBUG_OBJECT (sink, "%d streamheader buffers", buffers->len);
       for (i = 0; i < buffers->len; ++i) {
         GValue *bufval;
         GstBuffer *buffer;
@@ -1229,7 +1231,7 @@ gst_multi_fd_sink_client_queue_buffer (GstMultiFdSink * sink,
         bufval = &g_array_index (buffers, GValue, i);
         g_assert (G_VALUE_TYPE (bufval) == GST_TYPE_BUFFER);
         buffer = g_value_peek_pointer (bufval);
-        GST_LOG_OBJECT (sink,
+        GST_DEBUG_OBJECT (sink,
             "[fd %5d] queueing streamheader buffer of length %d",
             client->fd.fd, GST_BUFFER_SIZE (buffer));
         gst_buffer_ref (buffer);
@@ -1457,7 +1459,7 @@ find_limits (GstMultiFdSink * sink,
       if (time_max != -1 && first - time >= time_max)
         max_hit = TRUE;
     } else {
-      GST_DEBUG_OBJECT (sink, "No timestamp on buffer");
+      GST_LOG_OBJECT (sink, "No timestamp on buffer");
     }
     /* time is OK or unknown, check and increase if not enough bytes */
     if (bytes_min != -1) {
@@ -1551,6 +1553,8 @@ gst_multi_fd_sink_new_client (GstMultiFdSink * sink, GstTCPClient * client)
 
   GST_DEBUG_OBJECT (sink,
       "[fd %5d] new client, deciding where to start in queue", client->fd.fd);
+  GST_DEBUG_OBJECT (sink, "queue is currently %d buffers long",
+      sink->bufqueue->len);
   switch (client->sync_method) {
     case GST_SYNC_METHOD_LATEST:
       /* no syncing, we are happy with whatever the client is going to get */
@@ -2373,6 +2377,9 @@ gst_multi_fd_sink_render (GstBaseSink * bsink, GstBuffer * buf)
   if (!padcaps && !bufcaps)
     goto no_caps;
 
+  /* get IN_CAPS first, code below might mess with the flags */
+  in_caps = GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_IN_CAPS);
+
   /* stamp the buffer with previous caps if no caps set */
   if (!bufcaps) {
     if (!gst_buffer_is_metadata_writable (buf)) {
@@ -2397,9 +2404,13 @@ gst_multi_fd_sink_render (GstBaseSink * bsink, GstBuffer * buf)
     gst_buffer_ref (buf);
   }
 
-  in_caps = GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_IN_CAPS);
-
-  GST_LOG_OBJECT (sink, "received buffer %p, in_caps: %d", buf, in_caps);
+  GST_LOG_OBJECT (sink, "received buffer %p, in_caps: %s, offset %"
+      G_GINT64_FORMAT ", offset_end %" G_GINT64_FORMAT
+      ", timestamp %" GST_TIME_FORMAT ", duration %" GST_TIME_FORMAT,
+      buf, in_caps ? "yes" : "no", GST_BUFFER_OFFSET (buf),
+      GST_BUFFER_OFFSET_END (buf),
+      GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
+      GST_TIME_ARGS (GST_BUFFER_DURATION (buf)));
 
   /* if we get IN_CAPS buffers, but the previous buffer was not IN_CAPS,
    * it means we're getting new streamheader buffers, and we should clear
