@@ -40,6 +40,7 @@
 
 #include <string.h>
 #include <math.h>
+#include <stdlib.h>
 
 G_BEGIN_DECLS
 
@@ -79,7 +80,7 @@ typedef struct _GstXvImageSinkClass GstXvImageSinkClass;
  * @height: the height in pixels of Display @disp
  * @widthmm: the width in millimeters of Display @disp
  * @heightmm: the height in millimeters of Display @disp
- * @par: the pixel aspect ratio calculated from @width, @widthmm and @height, 
+ * @par: the pixel aspect ratio calculated from @width, @widthmm and @height,
  * @heightmm ratio
  * @use_xshm: used to known wether of not XShm extension is usable or not even
  * if the Extension is present
@@ -115,6 +116,8 @@ struct _GstXContext {
   gboolean use_xshm;
 
   XvPortID xv_port_id;
+  guint nb_adaptors;
+  gchar ** adaptors;
   gint im_format;
 
   GList *formats_list;
@@ -205,10 +208,11 @@ struct _GstXvImageBuffer {
  * @pool_lock: used to protect the buffer pool
  * @image_pool: a list of #GstXvImageBuffer that could be reused at next buffer
  * allocation call
- * @synchronous: used to store if XSynchronous should be used or not (for 
+ * @synchronous: used to store if XSynchronous should be used or not (for
  * debugging purpose only)
  * @keep_aspect: used to remember if reverse negotiation scaling should respect
  * aspect ratio
+ * @handle_events: used to know if we should handle select XEvents or not
  * @brightness: used to store the user settings for color balance brightness
  * @contrast: used to store the user settings for color balance contrast
  * @hue: used to store the user settings for color balance hue
@@ -224,30 +228,32 @@ struct _GstXvImageSink {
   GstVideoSink videosink;
 
   char *display_name;
+  guint adaptor_no;
 
   GstXContext *xcontext;
   GstXWindow *xwindow;
   GstXvImageBuffer *xvimage;
   GstXvImageBuffer *cur_image;
-  
+
   GThread *event_thread;
   gboolean running;
 
   gint fps_n;
   gint fps_d;
-  
+
   GMutex *x_lock;
   GMutex *flow_lock;
 
   /* object-set pixel aspect ratio */
   GValue *par;
-  
+
   GMutex *pool_lock;
   GSList *image_pool;
-  
+
   gboolean synchronous;
   gboolean keep_aspect;
-  
+  gboolean handle_events;
+
   gint brightness;
   gint contrast;
   gint hue;
